@@ -68,7 +68,7 @@ with_mock_api({
 })
 
 test_that("covidcast_meta raises error when API signals one", {
-  stub(covidcast_meta, ".request", "")
+  stub(covidcast_meta, ".request_meta", "")
 
   expect_error(covidcast_meta(),
                class = "covidcast_meta_fetch_failed")
